if (typeof module !== "undefined" && typeof exports !== "undefined" && module.exports === exports){
  module.exports = 'angular-dt-list';
}
(function(window, angular) {
	'use strict';
  angular.module('angular-dt-list',[])

	function DTListController($scope,$stateParams){
	  this.$onInit = function() {
	    var model = $scope.$ctrl.ngModel
	    model.items = []
	    model.$item = {}
	    if(typeof(model.pagination)==="undefined")
	      model.pagination = {per_page:10,current:1,total:0}

	    if(typeof(model.desc)==="undefined")
	      model.desc = false

	    // reorder the list based on col, or toggle desc
	    model.sort = function(col){
	      if(model.order==col)
	        model.desc = !model.desc
	      model.order = col
	      model.request()
	    }

	    // make a request via relay
	    model.get = function(params,postprocess){
	      model.relay.get(model.service,params,function(response){
          var rsp = response
          if(response.fmt && response.fmt=="DTR")
            rsp = response.obj
          model.items = rsp
          if(typeof(postprocess)!=="undefined")
  	        postprocess(model.items)
	      })
	    }

	    // create an item and add it to the list
	    model.create = function($item) {
	      model.items.push($item)
	      return $item
	    }

	    // submit an item, with optional callback
	    model.save = function(item,callback){
	      model.relay.save(model.service,item,function(response){
          var rsp = response
          if(response.fmt && response.fmt=="DTR")
            rsp = response.obj
	        for( var k in rsp ){
	          item[k] = rsp[k]
	        } //copy values into existing item
	        if(typeof(callback)!=="undefined")
	          callback(item)
	      })
	      return item
	    }

	    // remove an object from the list by index with optional callback
	    model.delete = function(idx,callback){
	      var r = confirm('Are you sure you want to delete this object?')
	      if(r==true){
	        var item = model.items[idx]
	        model.relay.delete(model.service,{id:item.id},function(){
	          model.items.splice(idx,1)
	          if(typeof(callback)!=="undefined")
	            callback(idx);
	        })
	      }
	    }

	    model.choose = function(item){
	      if(typeof(model.$item.id)!=="undefined")
	        model.save(model.$item)
	      model.$item = item
        return item
	    }

	    model.pageChanged = function(newPage) {
	        model.pagination.current = newPage
	        model.request();
	    };
	    model.getPage = function(params,postprocess){
	      model.relay.get(model.service,params,function(response){
          var rsp = response
          if(response.fmt && response.fmt=="DTR")
            rsp = response.obj
	        model.items = rsp
	        model.pagination.total = rsp.total
          if(typeof(postprocess)!=="undefined")
  	        postprocess(model.items)
	      })
	    }

	    var init = true
	    $scope.$watch('$ctrl.ngModel.like',function(newValue,oldValue){
	      if(!init)
	        model.request()
	      init = false
	    })

	    model.request = function(){
	      var params = {
	        dt_q:model.like,
	        dt_pg:model.pagination.current,
	        dt_ct:model.pagination.per_page,
	        dt_order:model.order,
	        dt_desc:model.desc
	      }
        //sprinkle in page and model filters
        angular.forEach($scope.$ctrl.filters,function(v,k){
          params[k] = v
        })
        angular.forEach(model.filters,function(v,k){
          params[k] = v
        })
	      if(typeof(model.preprocess)!=="undefined")
	        params = model.preprocess(params)
	      if(model.pagination.per_page>0)
	        model.getPage(params,model.postprocess)
	      else
	        model.get(params,model.postprocess)
	    }
	    // go get some results
	    model.request()
	  }
	}

	angular.module('angular-dt-list').component('dtList', {
	  controller: ['$scope','$stateParams',DTListController],
	  bindings: {
	    ngModel: '=',
      filters: '='
	  }
	})

})(window, window.angular);
